==> Epics.uv7.24.tar.gz: prog/epdeflection.f <==
      subroutine epdeflection(icon)
      implicit none
#include "ZepTrackv.h"
#include "ZepTrackp.h"
c             all deflections are managed here:
c     1) multiple Coulomb scattering
c     2) magnetic deflection
c     3) electric deflection
c
      integer icon  ! output.  icon=1 when infinite loop or unchanged

      record /epmove/moveSave
      integer nx

c          save current Move
      moveSave.Track.pos = Move.Track.pos

      if(Media(MediaNo).name .ne. 'sp' .and.
     *     Media(MediaNo).name .ne. 'hollow') then
         call epdefByMScat
      endif
      if(MagField .ne. 0) then
         call epdefByB(icon)
      endif
      if(ElecField .ne. 0) then
         call epdefByE
      endif
c       check if position moved to another component
c       due to scattering, if so, resotore the position
c       without  scattering/deflection
      if(.not. Move.Cross) then  ! ++++++++++
         call eppos2cn(Cn, Move.Track, nx)
         if(Cn .ne. nx) then
c           the new pos. dose not remain in the same component
c            no displacement due to scattering
            Move.Track.pos = moveSave.Track.pos
         endif
      endif
      end
c     ***********************
      subroutine epdefByMScat
c         deflection by Coulomb scattering
      implicit none
#include "ZepTrackv.h"
#include "ZepTrackp.h"

      record /epDirec/ dsa   ! dire ccos of scattering angle
      record /epDirec/ w
      real*8 theta, sint, cs, sn, tmp, avx, avy, disp
      real*8 r, dx, dy, g1, g2, gf1, gf2, beta2, tetarms

      call epmulScat(Media(MediaNo), theta) ! theta < pi/2
      if(theta .lt. 0.01d0) then
c                 cos
         dsa.z = 1.-theta**2/2
         sint = theta
      else
         dsa.z = cos(theta)
         sint = sin(theta)
      endif
c        azimuthal angle
      call kcossn(cs, sn)
c
      dsa.x = sint * cs
      dsa.y = sint * sn

      if( ( ALateCor .eq. 2 .or.
     *   (ALateCor .eq. 1 .and. .not. Moliere )) .and.
     *        .not. Move.Cross) then  !+++++++++++++
c         sample displacement correlated to theta 
c                 this is the same as P.D.B though look like
c               diff.
         tmp = Move.dl/2.d0
         avx = tmp * dsa.x
         avy = tmp * dsa.y
c                dispersion
         gf1 = cTrack.p.fm.p(4)/cTrack.p.mass
         gf2 = Move.Track.p.fm.p(4)/cTrack.p.mass
         beta2 = 1.d0 - 1.d0/gf1/gf2
         if(beta2 .le. 0.) then
            disp = 0.d0
         else
            if(Move.dt .gt. 1.d-3) then
c              beta is considered later in disp
               tetarms = Escat/cTrack.p.fm.p(4)*abs(cTrack.p.charge)*
     *      sqrt(Move.dt)*(1.0 + 0.038*log(Move.dt))
c     *      sqrt(Move.dt)

            else
               tetarms = Escat/cTrack.p.fm.p(4)*abs(cTrack.p.charge)*
     *              sqrt(Move.dt)
            endif
            disp=tetarms/sqrt(6.d0*beta2)*Move.dl/2.d0
c               sample 2 independent gaussian variables
c             with mean 0 and var 1
         endif
         call kgauss2(0.d0, 1.0d0, g1, g2)
         dx = g1 * disp + avx
         dy = g2 * disp + avy

c                  displacement

         r=sqrt(dx*dx+dy*dy)
c              direction cos of vector r in original sys.
         if(r .ne. 0.) then
            w.x = dx/r
            w.y = dy/r
            w.z = 0.
c                 transform w1,w2,w3 to original sys.
            call eptransVect(cTrack.w, w, w)
c               r is already in cm.
c              add scattering effect.
c              r*w is displacement by scattering
            Move.Track.pos.x = r*w.x + Move.Track.pos.x
            Move.Track.pos.y = r*w.y + Move.Track.pos.y
            Move.Track.pos.z = r*w.z + Move.Track.pos.z
         endif
      endif
c        convert scattering angle at end of path to
c        original system
      call eptransVect(Move.Track.w, dsa, Move.Track.w)
      end
c     ********************
      subroutine epdefByB(icon)
#include "ZepTrackp.h"
#include "ZepTrackv.h"
       integer icon  ! output. =1 if infinite loop or unchaged
       record /ep3Vec/ dispmr
       record /epDirec/ dispmd
       real*8  norm
       real*8 path, radius, loop
       data   path/0./
       save   path
c
c           trick to avoid semi-infinite loop in vaccum 
c
       if(Media(MediaNo).name .ne. 'sp') then
          path = 0.
       else
          if(path .eq. 0.) then
             call epmagDefR(cTrack, Bfield, radius) ! rough radius
c                 if  a ptcl runs  SyncLoop times of cyclotron  radius
c                 discard it
             loop = SyncLoop * radius
          endif
          path = path + Move.dl
          if(path .gt. loop) then
             Move.Trunc = .True.
             icon =1
             path = 0.
          endif
       endif

       call epmagneticDef(cTrack, Bfield, Move.dl, dispmr, dispmd)

c          displacement
c              ++++++++++++++
       if(.not. Move.Cross) then
          Move.Track.pos.x = Move.Track.pos.x + dispmr.x
          Move.Track.pos.y = Move.Track.pos.y + dispmr.y
          Move.Track.pos.z = Move.Track.pos.z + dispmr.z
       endif
c               direction
       Move.Track.w.x = Move.Track.w.x + dispmd.x
       Move.Track.w.y = Move.Track.w.y + dispmd.y
       Move.Track.w.z = Move.Track.w.z + dispmd.z
c           normalize
       norm = sqrt(Move.Track.w.x**2 +
     *            Move.Track.w.y**2 +
     *            Move.Track.w.z**2)

       Move.Track.w.x = Move.Track.w.x/norm
       Move.Track.w.y = Move.Track.w.y/norm
       Move.Track.w.z = Move.Track.w.z/norm

      end
      subroutine epdefByE
      end

c      
c     ************************************************  
      subroutine epmagneticDef(aTrack, B, len, dr,  dd)
c     ************************************************  
c      Compute magnetic deflection (angle and displacement)
c   Note that:
c      dr must be added  as  newpos = oldpos + len * olddir + dr
c      dd may be added  as  newdir = olddir + dd ; then normalize newdi.
c  This is a small modification of Cosmos function.
c     In Cosmos,  length is m, B is T
 
      implicit none
#include "ZepTrack.h"
#include "Zep3Vec.h"
 
      record /epTrack/ aTrack     ! input. a charged ptcl
      record /ep3Vec/ B            ! input. magnetic field vector. in T
                                  !  in local coordinate
      real*8  len         ! input. length  travelled in cm
      record /epPos/  dr  ! output. displacement vector in cm
      record /epDirec/dd  ! output. direction cos displacement.
c                 Note. This is approx that  beta * dbeta =0
c                 (orthogonal).  dd is dbeta/beta. and should be
c                 small enough.
      
      real*8   pabs, temp


      record /ep3Vec/ dxB    !  direction x B
      
      dxB.x = aTrack.w.y * B.z - 
     *    aTrack.w.z * B.y
      dxB.y = aTrack.w.z * B.x - 
     *    aTrack.w.x * B.z
      dxB.z = aTrack.w.x * B.y -
     *     aTrack.w.y * B.x

c      write(*, *) ' v x B', dxB.x, dxB.y, dxB.z

      call cpxyzp(aTrack.p.fm, pabs)   ! |p|
      if(pabs .eq. 0.) then
         temp = 0.
      else
c         temp = len**2 *0.3/2  * aTrack.p.charge /pabs ;  cosmos
c             ( len*10^-2) * 100.  makes len*10^-2
         temp = len**2 *0.3d-2 /2  * aTrack.p.charge /pabs  ! cm->m

      endif
      dr.x = dxB.x * temp
      dr.y = dxB.y * temp
      dr.z = dxB.z * temp

      if(pabs .gt. 0.) then
c         temp = 0.3 *len * aTrack.p.charge /pabs  ;   cosmos
          temp = 0.3d-2 *len * aTrack.p.charge /pabs  
      else
         temp = 0.
      endif
      dd.x = dxB.x * temp
      dd.y = dxB.y * temp
      dd.z = dxB.z * temp

      end

==> Epics.uv8.10.tar.gz: prog/epdeflection.f <==
      subroutine epdeflection(icon)
      implicit none
#include "ZepTrackv.h"
#include "ZepTrackp.h"
c             all deflections are managed here:
c     1) multiple Coulomb scattering
c     2) magnetic deflection
c     3) electric deflection
c
      integer icon  ! output.  icon=1 when infinite loop or unchanged

      record /epmove/moveSave
      integer nx

c          save current Move
      moveSave.Track.pos = Move.Track.pos

      if(Media(MediaNo).name .ne. 'sp' .and.
     *     Media(MediaNo).name .ne. 'hollow') then
         call epdefByMScat
      endif
      if(MagField .ne. 0) then
         call epdefByB(icon)
      endif
      if(ElecField .ne. 0) then
         call epdefByE
      endif
c       check if position moved to another component
c       due to scattering, if so, resotore the position
c       without  scattering/deflection
      if(.not. Move.Cross) then  ! ++++++++++
         call eppos2cn(Cn, Move.Track, nx)
         if(Cn .ne. nx) then
c           the new pos. dose not remain in the same component
c            no displacement due to scattering
            Move.Track.pos = moveSave.Track.pos
         endif
      endif
      end
c     ***********************
      subroutine epdefByMScat
c         deflection by Coulomb scattering
      implicit none
#include "ZepTrackv.h"
#include "ZepTrackp.h"

      record /epDirec/ dsa   ! dire ccos of scattering angle
      record /epDirec/ w
      real*8 theta, sint, cs, sn, tmp, avx, avy, disp
      real*8 r, dx, dy, g1, g2, gf1, gf2, beta2, tetarms

      call epmulScat(Media(MediaNo), theta) ! theta < pi/2
      if(theta .lt. 0.01d0) then
c                 cos
         dsa.z = 1.-theta**2/2
         sint = theta
      else
         dsa.z = cos(theta)
         sint = sin(theta)
      endif
c        azimuthal angle
      call kcossn(cs, sn)
c
      dsa.x = sint * cs
      dsa.y = sint * sn

      if( ( ALateCor .eq. 2 .or.
     *   (ALateCor .eq. 1 .and. .not. Moliere )) .and.
     *        .not. Move.Cross) then  !+++++++++++++
c         sample displacement correlated to theta 
c                 this is the same as P.D.B though look like
c               diff.
         tmp = Move.dl/2.d0
         avx = tmp * dsa.x
         avy = tmp * dsa.y
c                dispersion
         gf1 = cTrack.p.fm.p(4)/cTrack.p.mass
         gf2 = Move.Track.p.fm.p(4)/cTrack.p.mass
         beta2 = 1.d0 - 1.d0/gf1/gf2
         if(beta2 .le. 0.) then
            disp = 0.d0
         else
            if(Move.dt .gt. 1.d-3) then
c              beta is considered later in disp
               tetarms = Escat/cTrack.p.fm.p(4)*abs(cTrack.p.charge)*
     *      sqrt(Move.dt)*(1.0 + 0.038*log(Move.dt))
c     *      sqrt(Move.dt)

            else
               tetarms = Escat/cTrack.p.fm.p(4)*abs(cTrack.p.charge)*
     *              sqrt(Move.dt)
            endif
            disp=tetarms/sqrt(6.d0*beta2)*Move.dl/2.d0
c               sample 2 independent gaussian variables
c             with mean 0 and var 1
         endif
         call kgauss2(0.d0, 1.0d0, g1, g2)
         dx = g1 * disp + avx
         dy = g2 * disp + avy

c                  displacement

         r=sqrt(dx*dx+dy*dy)
c              direction cos of vector r in original sys.
         if(r .ne. 0.) then
            w.x = dx/r
            w.y = dy/r
            w.z = 0.
c                 transform w1,w2,w3 to original sys.
            call eptransVect(cTrack.w, w, w)
c               r is already in cm.
c              add scattering effect.
c              r*w is displacement by scattering
            Move.Track.pos.x = r*w.x + Move.Track.pos.x
            Move.Track.pos.y = r*w.y + Move.Track.pos.y
            Move.Track.pos.z = r*w.z + Move.Track.pos.z
         endif
      endif
c        convert scattering angle at end of path to
c        original system
      call eptransVect(Move.Track.w, dsa, Move.Track.w)
      end
c     ********************
      subroutine epdefByB(icon)
#include "ZepTrackp.h"
#include "ZepTrackv.h"
       integer icon  ! output. =1 if infinite loop or unchaged
       record /ep3Vec/ dispmr
       record /epDirec/ dispmd
       real*8  norm
       real*8 path, radius, loop
       data   path/0./
       save   path, loop
c
c           trick to avoid semi-infinite loop in vaccum 
c
c       if(Media(MediaNo).name .ne. 'sp') then
c          path = 0.
c       else
       if(pathInB .eq. 0.) then
          call epmagDefR(cTrack, Bfield, radius) ! rough radius
c                 if  a ptcl runs  SyncLoop times of cyclotron  radius
c                 discard it
          loop = SyncLoop * radius
       endif
       pathInB = pathInB + Move.dl
       if(pathInB .gt. loop) then
          Move.Trunc = .True.
          icon =1
          pathInB = 0.
       endif
c       endif

       call epmagneticDef(cTrack, Bfield, Move.dl, dispmr, dispmd)

c          displacement
c              ++++++++++++++
       if(.not. Move.Cross) then
          Move.Track.pos.x = Move.Track.pos.x + dispmr.x
          Move.Track.pos.y = Move.Track.pos.y + dispmr.y
          Move.Track.pos.z = Move.Track.pos.z + dispmr.z
       endif
c               direction
       Move.Track.w.x = Move.Track.w.x + dispmd.x
       Move.Track.w.y = Move.Track.w.y + dispmd.y
       Move.Track.w.z = Move.Track.w.z + dispmd.z
c           normalize
       norm = sqrt(Move.Track.w.x**2 +
     *            Move.Track.w.y**2 +
     *            Move.Track.w.z**2)

       Move.Track.w.x = Move.Track.w.x/norm
       Move.Track.w.y = Move.Track.w.y/norm
       Move.Track.w.z = Move.Track.w.z/norm

      end
      subroutine epdefByE
      end

c      
c     ************************************************  
      subroutine epmagneticDef(aTrack, B, len, dr,  dd)
c     ************************************************  
c      Compute magnetic deflection (angle and displacement)
c   Note that:
c      dr must be added  as  newpos = oldpos + len * olddir + dr
c      dd may be added  as  newdir = olddir + dd ; then normalize newdi.
c  This is a small modification of Cosmos function.
c     In Cosmos,  length is m, B is T
 
      implicit none
#include "ZepTrack.h"
#include "Zep3Vec.h"
 
      record /epTrack/ aTrack     ! input. a charged ptcl
      record /ep3Vec/ B            ! input. magnetic field vector. in T
                                  !  in local coordinate
      real*8  len         ! input. length  travelled in cm
      record /epPos/  dr  ! output. displacement vector in cm
      record /epDirec/dd  ! output. direction cos displacement.
c                 Note. This is approx that  beta * dbeta =0
c                 (orthogonal).  dd is dbeta/beta. and should be
c                 small enough.
      
      real*8   pabs, temp


      record /ep3Vec/ dxB    !  direction x B
      
      dxB.x = aTrack.w.y * B.z - 
     *    aTrack.w.z * B.y
      dxB.y = aTrack.w.z * B.x - 
     *    aTrack.w.x * B.z
      dxB.z = aTrack.w.x * B.y -
     *     aTrack.w.y * B.x

c      write(*, *) ' v x B', dxB.x, dxB.y, dxB.z

      call cpxyzp(aTrack.p.fm, pabs)   ! |p|
      if(pabs .eq. 0.) then
         temp = 0.
      else
c         temp = len**2 *0.3/2  * aTrack.p.charge /pabs ;  cosmos
c             ( len*10^-2) * 100.  makes len*10^-2
         temp = len**2 *0.3d-2 /2  * aTrack.p.charge /pabs  ! cm->m

      endif
      dr.x = dxB.x * temp
      dr.y = dxB.y * temp
      dr.z = dxB.z * temp

      if(pabs .gt. 0.) then
c         temp = 0.3 *len * aTrack.p.charge /pabs  ;   cosmos
          temp = 0.3d-2 *len * aTrack.p.charge /pabs  
      else
         temp = 0.
      endif
      dd.x = dxB.x * temp
      dd.y = dxB.y * temp
      dd.z = dxB.z * temp

      end
